==> Pcore-XMPP-v0.10.5.tar.gz: Pcore-XMPP-v0.10.5/lib/Pcore/Core/Event/Listener/xmpp.pm <==
package Pcore::Core::Event::Listener::xmpp;

use Pcore -class, -ansi, -const;
use Pcore::XMPP;
use Pcore::Util::Scalar qw[weaken];
use Pcore::Util::Data qw[to_json];
use Time::HiRes qw[];

with qw[Pcore::Core::Event::Listener];

has tmpl => '[<: $date.strftime("%Y-%m-%d %H:%M:%S.%4N") :>][<: $channel :>][<: $level :>] <: $title | raw :><: $text | raw :>';

has username => ( required => 1 );    # is => 'ro', isa => Str, required => 1
has password => ( required => 1 );    # isa => Str,         required => 1
has host     => ( required => 1 );    # isa => Str,         required => 1
has port => 5222;                     # isa => PositiveInt
has to => ( required => 1 );          # isa => Str,         required => 1

has _tmpl  => ( init_arg => undef );  # InstanceOf ['Pcore::Util::Tmpl'], init_arg => undef
has _h     => ( init_arg => undef );  # InstanceOf ['Pcore::Handle::xmpp'],   init_arg => undef
has _queue => ( init_arg => undef );  # isa => ArrayRef, init_arg => undef
has _init  => ( init_arg => undef );  #  isa => Bool, init_arg => undef

const our $INDENT => q[ ] x 4;

sub _build_id ($self) { return "xmpp://$self->{host}:$self->{port}?username=$self->{username}&to=$self->{to}" }

sub forward_event ( $self, $ev ) {

    # init
    if ( !$self->{_init} ) {
        $self->{_init} = 1;

        # init template
        $self->{_tmpl} = P->tmpl;

        $self->{_tmpl}->add_tmpl( message => $self->{tmpl} );

        $self->{_h} = AnyEvent::XMPP::Client->new;

        $self->{_h}->add_account( $self->{username}, $self->{password}, $self->{host}, $self->{port}, undef );

        weaken $self;

        $self->{_h}->reg_cb(
            session_ready => sub ( $h, $acc ) {
                $self->_on_connect( $h, $acc ) if defined $self;

                return;
            }
        );

        $self->{_h}->start;
    }

    # sendlog
    {
        # prepare date object
        local $ev->{date} = P->date->from_epoch( $ev->{timestamp} // Time::HiRes::time() );

        # prepare text
        local $ev->{text};

        if ( defined $ev->{data} ) {

            # serialize reference
            $ev->{text} = $LF . ( ref $ev->{data} ? to_json( $ev->{data}, readable => 1 )->$* : $ev->{data} );

            # indent
            $ev->{text} =~ s/^/$INDENT/smg;

            # remove all trailing "\n"
            local $/ = '';

            chomp $ev->{text};
        }

        my $message = $self->{_tmpl}->render( 'message', $ev );

        if ( !$self->{_h}->get_connected_accounts ) {
            push $self->{_queue}->@*, $message;
        }
        else {
            $self->{_h}->send_message( $message->$*, $self->{to}, $self->{username}, 'chat' );
        }
    }

    return;
}

sub _on_connect ( $self, $xmpp, $acc ) {
    while ( my $msg = shift $self->{_queue}->@* ) {
        $xmpp->send_message( $msg->$*, $self->{to}, $self->{username}, 'chat' );
    }

    return;
}

1;
## -----SOURCE FILTER LOG BEGIN-----
##
## PerlCritic profile "pcore-script" policy violations:
## +------+----------------------+----------------------------------------------------------------------------------------------------------------+
## | Sev. | Lines                | Policy                                                                                                         |
## |======+======================+================================================================================================================|
## |    3 | 62                   | Variables::RequireInitializationForLocalVars - "local" variable not initialized                                |
## |------+----------------------+----------------------------------------------------------------------------------------------------------------|
## |    2 | 59, 62               | Variables::ProhibitLocalVars - Variable declared as "local"                                                    |
## |------+----------------------+----------------------------------------------------------------------------------------------------------------|
## |    2 | 73                   | ValuesAndExpressions::ProhibitEmptyQuotes - Quotes used with a string containing no non-whitespace characters  |
## |------+----------------------+----------------------------------------------------------------------------------------------------------------|
## |    1 | 11                   | ValuesAndExpressions::RequireInterpolationOfMetachars - String *may* require interpolation                     |
## +------+----------------------+----------------------------------------------------------------------------------------------------------------+
##
## -----SOURCE FILTER LOG END-----
__END__
=pod

=encoding utf8

=head1 NAME

Pcore::Core::Event::Listener::xmpp

=head1 SYNOPSIS

    P->bind_events(
        'LOG.#',
        [   'xmpp:',
            username => 'user@gmail.com',
            password => '',
            host     => 'talk.google.com',
            port     => 5222,
            to       => 'target@domain.net',
        ]
    );

=head1 DESCRIPTION

=head1 ATTRIBUTES

=head1 METHODS

=head1 SEE ALSO

=cut

==> Pcore-XMPP-v0.10.6.tar.gz: Pcore-XMPP-v0.10.6/lib/Pcore/Core/Event/Listener/xmpp.pm <==
package Pcore::Core::Event::Listener::xmpp;

use Pcore -class, -ansi, -const;
use Pcore::XMPP;
use Pcore::Util::Scalar qw[weaken];
use Pcore::Util::Data qw[to_json];
use Time::HiRes qw[];

with qw[Pcore::Core::Event::Listener];

has tmpl => '[<: $date.strftime("%Y-%m-%d %H:%M:%S.%4N") :>][<: $channel :>][<: $level :>] <: $title | raw :><: $text | raw :>';

has username => ( required => 1 );    # Str
has password => ( required => 1 );    # Str
has host     => ( required => 1 );    # Str
has port => 5222;                     # PositiveInt
has to => ( required => 1 );          # Str

has _tmpl  => ( init_arg => undef );  # InstanceOf ['Pcore::Util::Tmpl']
has _h     => ( init_arg => undef );  # InstanceOf ['Pcore::Handle::xmpp']
has _queue => ( init_arg => undef );  # ArrayRef
has _init  => ( init_arg => undef );  # Bool

const our $INDENT => q[ ] x 4;

sub _build_id ($self) { return "xmpp://$self->{host}:$self->{port}?username=$self->{username}&to=$self->{to}" }

sub forward_event ( $self, $ev ) {

    # init
    if ( !$self->{_init} ) {
        $self->{_init} = 1;

        # init template
        $self->{_tmpl} = P->tmpl;

        $self->{_tmpl}->add_tmpl( message => $self->{tmpl} );

        $self->{_h} = AnyEvent::XMPP::Client->new;

        $self->{_h}->add_account( $self->{username}, $self->{password}, $self->{host}, $self->{port}, undef );

        weaken $self;

        $self->{_h}->reg_cb(
            session_ready => sub ( $h, $acc ) {
                $self->_on_connect( $h, $acc ) if defined $self;

                return;
            }
        );

        $self->{_h}->start;
    }

    # sendlog
    {
        # prepare date object
        local $ev->{date} = P->date->from_epoch( $ev->{timestamp} // Time::HiRes::time() );

        # prepare text
        local $ev->{text};

        if ( defined $ev->{data} ) {

            # serialize reference
            $ev->{text} = $LF . ( ref $ev->{data} ? to_json( $ev->{data}, readable => 1 )->$* : $ev->{data} );

            # indent
            $ev->{text} =~ s/^/$INDENT/smg;

            # remove all trailing "\n"
            local $/ = '';

            chomp $ev->{text};
        }

        my $message = $self->{_tmpl}->render( 'message', $ev );

        if ( !$self->{_h}->get_connected_accounts ) {
            push $self->{_queue}->@*, $message;
        }
        else {
            $self->{_h}->send_message( $message->$*, $self->{to}, $self->{username}, 'chat' );
        }
    }

    return;
}

sub _on_connect ( $self, $xmpp, $acc ) {
    while ( my $msg = shift $self->{_queue}->@* ) {
        $xmpp->send_message( $msg->$*, $self->{to}, $self->{username}, 'chat' );
    }

    return;
}

1;
## -----SOURCE FILTER LOG BEGIN-----
##
## PerlCritic profile "pcore-script" policy violations:
## +------+----------------------+----------------------------------------------------------------------------------------------------------------+
## | Sev. | Lines                | Policy                                                                                                         |
## |======+======================+================================================================================================================|
## |    3 | 62                   | Variables::RequireInitializationForLocalVars - "local" variable not initialized                                |
## |------+----------------------+----------------------------------------------------------------------------------------------------------------|
## |    2 | 59, 62               | Variables::ProhibitLocalVars - Variable declared as "local"                                                    |
## |------+----------------------+----------------------------------------------------------------------------------------------------------------|
## |    2 | 73                   | ValuesAndExpressions::ProhibitEmptyQuotes - Quotes used with a string containing no non-whitespace characters  |
## |------+----------------------+----------------------------------------------------------------------------------------------------------------|
## |    1 | 11                   | ValuesAndExpressions::RequireInterpolationOfMetachars - String *may* require interpolation                     |
## +------+----------------------+----------------------------------------------------------------------------------------------------------------+
##
## -----SOURCE FILTER LOG END-----
__END__
=pod

=encoding utf8

=head1 NAME

Pcore::Core::Event::Listener::xmpp

=head1 SYNOPSIS

    P->bind_events(
        'LOG.#',
        [   'xmpp:',
            username => 'user@gmail.com',
            password => '',
            host     => 'talk.google.com',
            port     => 5222,
            to       => 'target@domain.net',
        ]
    );

=head1 DESCRIPTION

=head1 ATTRIBUTES

=head1 METHODS

=head1 SEE ALSO

=cut
